from __future__ import annotations

from itertools import chain
import os
from pathlib import Path

from ..store import Store

__all__ = ['find_node_modules', 'find_node_modules_binary']


def find_node_modules() -> Path | None:
    store = Store()

    for path in chain((store.working_directory,), store.working_directory.parents):
        if (result := path / 'node_modules').is_dir():
            return result


def find_node_modules_binary(name: str) -> Path | None:
    node_modules = find_node_modules()
    if node_modules is None:
        return None

    if is_executable(result := node_modules / '.bin' / name):
        return result


def is_executable(path: Path) -> bool:
    return os.access(path, os.F_OK | os.X_OK)
